const CardService = require('./CardService');
const DataManagement = require('./DataManagement');
const CostReview = require('./CostReview');
const AnalizaCardapio = require('./AnalizaCardapio');
const MenuAnalytics = require('./Helper/MenuAnalytics');
const Calendar = require('./Helper/Calendar');
const FlexRules = require('./Helper/FlexRules');
const MenuReview = require('./Helper/MenuReview');
this.prepareCardForInsertion = (cardapio) => {
  cardapio.datas = cardapio.datas.filter((d) => d.comensais != 0);
  cardapio.datas.forEach((dia) => {
    if (dia.pratos) {
      dia.pratos.forEach((prato) => {
        prato.unidadePrato =
          prato.prato && prato.prato[0] == 'P'
            ? prato.prato.substr(0, 1)
            : cardapio.unidade;
        prato.codprato = prato.prato;
        prato.prato =
          prato.unidadePrato == 'P'
            ? (prato.prato || '').substr(1)
            : (prato.prato || '').replace(cardapio.unidade, '');
        prato.nrPratoCardapio = prato.sequencial;
        prato.tipoPrato = prato.COD_TIP_PRATO;
      });
    }
  });
  return cardapio;
};

this.genCardapio = (rawData, metaPar) => {
  DataManagement.testRawData(rawData);
  MenuAnalytics.clear();
  Calendar.clear();
  const { P_DTMES, CDOPERADOR } = rawData;

  const { COD_FILIAL, COD_SERVICO, VLR_CUSTO_MIN, VRL_CUSTO_MAX } =
    rawData.configuracao_servico.data[0];

  let unidade = COD_FILIAL;
  let servico = COD_SERVICO;
  let meta = { VLR_CUSTO_MIN, VRL_CUSTO_MAX };

  if (metaPar.meta) {
    meta.VLR_CUSTO_MIN = parseFloat(metaPar.metaMin);
    meta.VRL_CUSTO_MAX = parseFloat(metaPar.metaMax);
  }

  let cor = rawData.grupo_tipo_prato.data[0].PERC_GRP_TIP_PRATO / 100;
  let operador = CDOPERADOR;
  let mes = P_DTMES;

  let cardapio = {
    unidade,
    servico,
    operador,
    mes,
  };

  let cardCong = DataManagement.setCardCong(rawData.cong_days.data);
  let lastWeekProds = DataManagement.setCardCong(rawData.lastWeekProds.data);

  let srvConf = rawData.configuracao_servico.data;
  let seqOrder = rawData.seq_order.data;
  let conjuntoPorcao = rawData.conjunto_porcao.data;
  let restricaoIngrediente = rawData.restricao_de_repeticao_ingrediente.data;

  let prcConsumoPrato = rawData.prc_consumo_prato.data;

  let tpCorte = rawData.tp_corte.data;
  let mdPreparo = rawData.md_preparo.data;
  let ingPrinc = rawData.ing_princ.data;

  let calendario = DataManagement.setDatePropsOnCalendario(
    rawData.calendario.data
  );
  let receitaPrato = rawData.receita_prato.data;
  let grupoPrato = rawData.grupo_prato.data;

  let utilizacaoPratoSemana = rawData.utilizacao_prato_semana.data;
  let res_grupo_prato = rawData.res_grupo_prato.data;

  let managedServConfig = DataManagement.manageSeqSrvConf(srvConf);
  let configWithPlates = DataManagement.addPlatesAtSrvConf(
    managedServConfig,
    grupoPrato
  );
  let cardapiodeCanditados = DataManagement.fillCardWithPlates(
    receitaPrato,
    cardapio,
    configWithPlates,
    calendario,
    utilizacaoPratoSemana,
    prcConsumoPrato,
    managedServConfig
  );
  cardapio = DataManagement.filterCandidate(
    cardapiodeCanditados,
    receitaPrato,
    res_grupo_prato
  );

  cardapiodeCanditados = null;
  managedServConfig = null;
  configWithPlates = null;
  res_grupo_prato = null;

  let frqGrupoPratoMes = rawData.frq_grupo_prato_mes.data;
  let frqPratoMes = rawData.frq_prato_mes.data;
  let frqProdGrupoPrato = rawData.frq_prod_grupo_prato.data;
  let frequencias = [].concat(frqGrupoPratoMes, frqPratoMes, frqProdGrupoPrato);

  frqGrupoPratoMes = null;
  frqPratoMes = null;
  frqProdGrupoPrato = null;

  let res_prato_combinacao = rawData.res_prato_combinacao.data;
  let prato_combinacao = rawData.prato_combinacao.data;
  prato_combinacao = DataManagement.ajustSeq(
    prato_combinacao,
    frequencias,
    grupoPrato
  );
  let complexidade = rawData.complexidade.data;

  let sequencialCenarioOriginal = rawData.sequencialCenarioOriginal.data;
  let parsedCardapio = rawData.parsedCardapio;
  if(parsedCardapio === ''){
    sequencialCenarioOriginal = null;
  }
    let diasCardOrigem = DataManagement.getPratosFromServOrigem(
    sequencialCenarioOriginal,
    parsedCardapio
  );

  cardapio = DataManagement.setProdsOnPratos(cardapio, receitaPrato);
  cardapio = DataManagement.filterExcludedCarac(
    cardapio,
    tpCorte,
    mdPreparo,
    ingPrinc
  );
  cardapio = CardService.generateTonsOfCards(
    utilizacaoPratoSemana,
    cor,
    cardapio,
    meta,
    prato_combinacao,
    servico,
    res_prato_combinacao,
    complexidade,
    frequencias,
    calendario,
    cardCong,
    lastWeekProds,
    conjuntoPorcao,
    grupoPrato,
    prcConsumoPrato,
    restricaoIngrediente,
    diasCardOrigem,
    sequencialCenarioOriginal
  );

  // APENAS RETORNO PARA O BACKOFFICE
  let vrmeta = metaPar.meta || rawData.configuracao_servico.data[0].VRCUSTPADR;
  let cardapioToAval = Object.assign({}, cardapio);
  cardapioToAval = DataManagement.genCard(
    cardapio,
    cardapioToAval,
    seqOrder,
    grupoPrato,
    receitaPrato,
    vrmeta,
    utilizacaoPratoSemana,
    conjuntoPorcao,
    rawData.regras_flexiveis,
  );
  cardapioToAval.frequencias = frequencias;
  cardapioToAval.meta = meta;
  cardapioToAval.analytics = MenuAnalytics.data;
  utilizacaoPratoSemana = null;

  return cardapioToAval;
};

let generateLogInfo = function (card) {
  let dias = card.tabela.pratos;

  let diasComBuracos = dias.filter((d) =>
    d.find((p) => !p.prato && p.otherOptions && p.otherOptions.length != 0)
  );
  diasComBuracos.forEach((d, key) => {
    diasComBuracos[key] = {
      dia: card.tabela.comensais_datas[dias.indexOf(d)][1],
      pratos: d,
    };
  });

  let diasComIngRepetidos = dias.filter((d) => {
    let ingToday = [];

    for (let i = 0; i < d.length; i++) {
      let p = d[i];
      if (p.prato) {
        if (p.INGR_PRINCIPAL_1 != null) {
          if (!ingToday.includes(p.INGR_PRINCIPAL_1)) {
            ingToday.push(p.INGR_PRINCIPAL_1);
          } else {
            return d;
          }
        }

        if (p.INGR_PRINCIPAL_2 != null) {
          if (!ingToday.includes(p.INGR_PRINCIPAL_2)) {
            ingToday.push(p.INGR_PRINCIPAL_2);
          } else {
            return d;
          }
        }
      }
    }
  });

  diasComIngRepetidos.forEach((d, key) => {
    diasComIngRepetidos[key] = {
      dia: card.tabela.comensais_datas[dias.indexOf(d)][1],
      pratos: d,
    };
  });

  return {
    diasComBuracos,
    diasComIngRepetidos,
  };
};

const toFixed = (value, decimalPlaces = 5) =>
  parseFloat(value.toFixed(decimalPlaces));

const reviewCard = (perfectCard, originalData) => {
  let oldDiffCost = null;

  let reviewCard = {
    oldPuAcum: toFixed(perfectCard.pu_aculm),
    puAcum: null,
    upgrade: null,
    upgradePerc: null,
    days: [],
    meta: perfectCard.meta,
    changes: 0,
  };
  if (perfectCard.dif_custo < 0) {
    let logChanges = {};
    let dishToIgnoreChange = new Map();
    do {
      oldDiffCost = perfectCard.dif_custo;
      reviewCard.changes += CostReview.analizeGeneratedCard(
        perfectCard,
        originalData,
        logChanges,
        dishToIgnoreChange
      );
    } while (perfectCard.dif_custo < 0 && oldDiffCost != perfectCard.dif_custo);
    reviewCard.days = logChanges;
  }
  reviewCard.puAcum = toFixed(perfectCard.pu_aculm);
  reviewCard.upgrade = toFixed(reviewCard.oldPuAcum - reviewCard.puAcum);
  reviewCard.upgradePerc = `${toFixed(
    ((reviewCard.oldPuAcum - reviewCard.puAcum) * 100) / reviewCard.oldPuAcum,
    2
  )}%`;
  if (perfectCard && perfectCard.tabela && perfectCard.tabela.log) {
    perfectCard.tabela.log.reviewCard = reviewCard;
  }
};

this.generatePerfectCard = function (data, meta) {
  let perfectCard = null;
  let cards = [];
  let trie = 0;
  let maxTry = 25;
  if(!data.regras_flexiveis) {
    data.regras_flexiveis = {
      data: [],
    }
  }
  FlexRules.setRules(data.regras_flexiveis.data);
  let calendar = [];
  do {
    trie++;
    let originalData = JSON.parse(JSON.stringify({ data, meta }));
    let cardapio = this.genCardapio(originalData.data, originalData.meta);
    calendar = originalData.data.calendario.data;
    cardapio.P_VRCUSTPADR = data.P_VRCUSTPADR;
    let comBuraco = cardapio.tabela.pratos.filter((d) =>
      d.find((p) => !p.prato && p.otherOptions && p.otherOptions.length != 0)
    ).length;
    cardapio.comBuraco = comBuraco;
    cardapio.trie = trie;

    cards.push(cardapio);
    if (comBuraco == 0) {
      perfectCard = cardapio;
    } else if (comBuraco == 1 && trie > Math.floor(maxTry / 2)) {
      perfectCard = null;
      trie = maxTry + 1;
    }
  } while (!perfectCard && trie < maxTry);

  let originalData = JSON.parse(JSON.stringify({ data, meta }));
  const restComb = originalData.data.res_prato_combinacao.data;
  const utilizationGroupWeek = originalData.data.utilizacao_prato_semana.data;
  if (!perfectCard) {
    let menorBuraco = 99;
    cards.forEach((c) => {
      if (c.comBuraco < menorBuraco) {
        menorBuraco = c.comBuraco;
      }
    });

    cards = cards
      .filter((c) => c.comBuraco == menorBuraco)
      .sort(function (a, b) {
        if (a.pu_aculm > b.pu_aculm) {
          return 1;
        }
        return -1;
      });
    perfectCard = cards[0];
    // AnalizaCardapio.execute(perfectCard, originalData.data);
    MenuReview.execute(perfectCard, { restComb, calendar, utilizationGroupWeek });
    // reviewCard(perfectCard, originalData);
    perfectCard.aditionalLogInfo = generateLogInfo(perfectCard);
  } else {
    MenuReview.execute(perfectCard, { restComb, calendar, utilizationGroupWeek });
    // AnalizaCardapio.execute(perfectCard, originalData.data);
    // reviewCard(perfectCard, originalData);
  }

  perfectCard.cardapio = this.prepareCardForInsertion(perfectCard.cardapio);

  return {
    perfectCard,
    trie,
  };
};
